@@ -1,7 +1,7 @@
 //! The parser: turns a series of tokens into an AST
 
 use std::{
-    collections::{HashSet, VecDeque},
+    collections::{HashSet, HashMap, VecDeque},
     fmt,
 };
 
@@ -28,22 +28,34 @@ pub enum ParseError {
     UnexpectedExtra(TextRange),
     /// UnexpectedWanted is used when specific tokens are expected, but different one is found
     UnexpectedWanted(SyntaxKind, TextRange, Box<[SyntaxKind]>),
-    /// UnexpectedWanted is used when a pattern is bound twice
+    /// UnexpectedDoubleBind is used when a pattern is bound twice
     UnexpectedDoubleBind(TextRange),
     /// UnexpectedEOF is used when the end of file is reached, while tokens are still expected
     UnexpectedEOF,
-    /// UnexpectedWanted is used when specific tokens are expected, but the end of file is reached
+    /// UnexpectedEOFWanted is used when specific tokens are expected, but the end of file is reached
     UnexpectedEOFWanted(Box<[SyntaxKind]>),
+    /// DuplicatedArgs is used when formal arguments are duplicated, e.g. `{ a, a }`
+    DuplicatedArgs(TextRange, String),
 }
 
 impl fmt::Display for ParseError {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         match self {
             ParseError::Unexpected(range) => {
-                write!(f, "error node at {}..{}", usize::from(range.start()), usize::from(range.end()))
+                write!(
+                    f,
+                    "error node at {}..{}",
+                    usize::from(range.start()),
+                    usize::from(range.end())
+                )
             }
             ParseError::UnexpectedExtra(range) => {
-                write!(f, "unexpected token at {}..{}", usize::from(range.start()), usize::from(range.end()))
+                write!(
+                    f,
+                    "unexpected token at {}..{}",
+                    usize::from(range.start()),
+                    usize::from(range.end())
+                )
             }
             ParseError::UnexpectedWanted(got, range, kinds) => write!(
                 f,
@@ -54,12 +66,26 @@ impl fmt::Display for ParseError {
                 kinds
             ),
             ParseError::UnexpectedDoubleBind(range) => {
-                write!(f, "unexpected double bind at {}..{}", usize::from(range.start()), usize::from(range.end()))
+                write!(
+                    f,
+                    "unexpected double bind at {}..{}",
+                    usize::from(range.start()),
+                    usize::from(range.end())
+                )
             }
             ParseError::UnexpectedEOF => write!(f, "unexpected end of file"),
             ParseError::UnexpectedEOFWanted(kinds) => {
                 write!(f, "unexpected end of file, wanted any of {:?}", kinds)
             }
+            ParseError::DuplicatedArgs(range, ident) => {
+                write!(
+                    f,
+                    "argument `{}` is duplicated in {}..{}",
+                    ident,
+                    usize::from(range.start()),
+                    usize::from(range.end())
+                )
+            }
         }
     }
 }
@@ -186,20 +212,26 @@ where
         self.get_text_position()
     }
     fn bump(&mut self) {
-        let next = self.buffer.pop_front().or_else(|| self.iter.next());
+        let next = self.try_next();
         match next {
             Some((token, s)) => {
                 if token.is_trivia() {
                     self.trivia_buffer.push((token, s))
                 } else {
-                    self.drain_trivia_buffer();
-                    self.consumed += TextSize::of(s.as_str());
-                    self.builder.token(NixLanguage::kind_to_raw(token), s.as_str())
+                    self.manual_bump(s, token);
                 }
             }
             None => self.errors.push(ParseError::UnexpectedEOF),
         }
     }
+    fn try_next(&mut self) -> Option<(SyntaxKind, SmolStr)> {
+        self.buffer.pop_front().or_else(|| self.iter.next())
+    }
+    fn manual_bump(&mut self, s: SmolStr, token: SyntaxKind) {
+        self.drain_trivia_buffer();
+        self.consumed += TextSize::of(s.as_str());
+        self.builder.token(NixLanguage::kind_to_raw(token), s.as_str())
+    }
     fn peek_data(&mut self) -> Option<&(SyntaxKind, SmolStr)> {
         while self.peek_raw().map(|&(t, _)| t.is_trivia()).unwrap_or(false) {
             self.bump();
@@ -311,6 +343,7 @@ where
         if self.peek().map(|t| t == TOKEN_CURLY_B_CLOSE).unwrap_or(true) {
             self.bump();
         } else {
+            let mut args = HashMap::<SmolStr, TextSize>::new();
             loop {
                 match self.expect_peek_any(&[TOKEN_CURLY_B_CLOSE, TOKEN_ELLIPSIS, TOKEN_IDENT]) {
                     Some(TOKEN_CURLY_B_CLOSE) => {
@@ -324,8 +357,28 @@ where
                     }
                     Some(TOKEN_IDENT) => {
                         self.start_node(NODE_PAT_ENTRY);
+                        let tp = self.get_text_position();
+                        let mut ident_done = false;
+                        if let Some((_, ident_name)) = self.peek_raw() {
+                            let contains = args.contains_key(ident_name);
+                            let id = ident_name.clone();
+                            let id_str = ident_name.to_string().clone();
+                            args.insert(id, tp);
+                            if contains {
+                                ident_done = true;
+                                self.start_error_node();
+                                self.expect_ident();
+                                let fin = self.finish_error_node();
+                                self.errors.push(ParseError::DuplicatedArgs(
+                                    TextRange::new(tp, fin),
+                                    id_str
+                                ));
+                            }
+                        }
 
-                        self.expect_ident();
+                        if !ident_done {
+                            self.expect_ident();
+                        }
 
                         if let Some(TOKEN_QUESTION) = self.peek() {
                             self.bump();
@@ -376,10 +429,18 @@ where
                         self.finish_node();
                     }
 
-                    while self.peek() != Some(TOKEN_SEMICOLON) {
-                        self.parse_val();
-                        if self.errors.last().map_or(false, |x| *x == ParseError::UnexpectedEOF) {
-                            break;
+                    loop {
+                        match self.peek() {
+                            Some(t) if t != TOKEN_SEMICOLON => {
+                                self.next_attr();
+                            }
+                            Some(_) => {
+                                break;
+                            }
+                            None => {
+                                self.errors.push(ParseError::UnexpectedEOF);
+                                break;
+                            }
                         }
                     }
 
@@ -484,6 +545,34 @@ where
             }
             TOKEN_DYNAMIC_START => self.parse_dynamic(),
             TOKEN_STRING_START => self.parse_string(),
+            TOKEN_PATH => {
+                let next = self.try_next();
+                if let Some((token, s)) = next {
+                    let is_complex_path = self
+                        .peek()
+                        .map_or(false, |t| t == TOKEN_INTERPOL_START);
+                    self.start_node(if is_complex_path { NODE_PATH_WITH_INTERPOL } else { NODE_LITERAL });
+                    self.manual_bump(s, token);
+                    if is_complex_path {
+                        loop {
+                            match self.peek_raw().map(|(t, _)| t) {
+                                Some(TOKEN_PATH) => self.bump(),
+                                Some(TOKEN_INTERPOL_START) => {
+                                    self.start_node(NODE_STRING_INTERPOL);
+                                    self.bump();
+                                    self.parse_expr();
+                                    self.expect(TOKEN_INTERPOL_END);
+                                    self.finish_node();
+                                },
+                                _ => break
+                            }
+                        }
+                    }
+                    self.finish_node();
+                } else {
+                    self.errors.push(ParseError::UnexpectedEOF);
+                }
+            },
             t if t.is_literal() => {
                 self.start_node(NODE_LITERAL);
                 self.bump();
@@ -727,7 +816,7 @@ where
 mod tests {
     use super::*;
 
-    use std::{ffi::OsStr, fmt::Write, fs, path::PathBuf};
+    use std::{env, ffi::OsStr, fmt::Write, fs, io::Write as IoWrite, path::PathBuf};
 
     #[test]
     fn whitespace_attachment_for_incomplete_code1() {
@@ -798,6 +887,7 @@ NODE_ROOT 0..5 {
     }
 
     fn test_dir(name: &str) {
+        let should_update = env::var("UPDATE_TESTS").map(|s| s == "1").unwrap_or(false);
         let dir: PathBuf = ["test_data", name].iter().collect();
 
         for entry in dir.read_dir().unwrap() {
@@ -812,6 +902,9 @@ NODE_ROOT 0..5 {
             }
             let ast = crate::parse(&code);
             path.set_extension("expect");
+            if !path.exists() {
+                fs::File::create(&path).expect("Failed to create .expect file");
+            }
             let expected = fs::read_to_string(&path).unwrap();
 
             let mut actual = String::new();
@@ -819,6 +912,12 @@ NODE_ROOT 0..5 {
                 writeln!(actual, "error: {}", error).unwrap();
             }
             writeln!(actual, "{}", ast.root().dump()).unwrap();
+            if should_update {
+                let mut file =
+                    fs::OpenOptions::new().write(true).truncate(true).open(&path).unwrap();
+                write!(file, "{}", actual).unwrap();
+                continue;
+            }
 
             if actual != expected {
                 path.set_extension("nix");
@@ -836,9 +935,9 @@ NODE_ROOT 0..5 {
     #[rustfmt::skip]
     mod dir_tests {
         use super::test_dir;
-        #[test] 
-        fn general() { 
-            test_dir("general"); 
+        #[test]
+        fn general() {
+            test_dir("general");
         }
 
         #[test]
