@@ -211,8 +211,17 @@ pub trait Wrapper: TypedNode {
     }
 }
 
+#[derive(Debug, Copy, Clone)]
 pub struct ParsedTypeError(pub SyntaxKind);
 
+impl fmt::Display for ParsedTypeError {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        writeln!(f, "invalid cast from kind {:?}", self.0)
+    }
+}
+
+impl std::error::Error for ParsedTypeError {}
+
 #[derive(Clone, Debug)]
 pub enum ParsedType {
     Apply(Apply),
@@ -242,6 +251,7 @@ pub enum ParsedType {
     UnaryOp(UnaryOp),
     Value(Value),
     With(With),
+    PathWithInterpol(PathWithInterpol),
 }
 
 impl TryFrom<SyntaxNode> for ParsedType {
@@ -276,6 +286,7 @@ impl TryFrom<SyntaxNode> for ParsedType {
             NODE_UNARY_OP => Ok(ParsedType::UnaryOp(UnaryOp::cast(node).unwrap())),
             NODE_LITERAL => Ok(ParsedType::Value(Value::cast(node).unwrap())),
             NODE_WITH => Ok(ParsedType::With(With::cast(node).unwrap())),
+            NODE_PATH_WITH_INTERPOL => Ok(ParsedType::PathWithInterpol(PathWithInterpol::cast(node).unwrap())),
             other => Err(ParsedTypeError(other)),
         }
     }
@@ -311,6 +322,7 @@ impl TypedNode for ParsedType {
             ParsedType::UnaryOp(n) => n.node(),
             ParsedType::Value(n) => n.node(),
             ParsedType::With(n) => n.node(),
+            ParsedType::PathWithInterpol(n) => n.node(),
         }
     }
 
@@ -428,8 +440,8 @@ typed! [
             nth!(self; 0)
         }
         /// Return the operator
-        pub fn operator(&self) -> BinOpKind {
-            self.first_token().and_then(|t| BinOpKind::from_token(t.kind())).expect("invalid ast")
+        pub fn operator(&self) -> Option<BinOpKind> {
+            self.first_token().and_then(|t| BinOpKind::from_token(t.kind()))
         }
         /// Return the right hand side of the binary operation
         pub fn rhs(&self) -> Option<SyntaxNode> {
@@ -530,5 +542,6 @@ typed! [
         pub fn body(&self) -> Option<SyntaxNode> {
             nth!(self; 1)
         }
-    }
+    },
+    NODE_PATH_WITH_INTERPOL => PathWithInterpol: Wrapper
 ];
